import pgzrun

import numpy as np

from pw26flow.game import InventoryItem, GameState, CONNECT_DISTANCE, TILE_SIZE
from pw26flow.maps import maps, FLOW_TEXTURES, TEXTURES, ANIMALS, START_NODE, GOAL


WIDTH = 800
HEIGHT = 600

LEVELS = [maps.flow00, maps.flow01]
current_level = 0


game = GameState(WIDTH, HEIGHT)
game.load_map(LEVELS[current_level])


game.active_inventory = InventoryItem(
    "connection", line=np.array([[-25, 25], [25, -25]]), targets=[None, None]
)


def on_mouse_move(pos):
    game.cursor_pos = np.array(pos)


def on_mouse_down(pos, button):
    if button == mouse.LEFT and game.active_inventory is not None:
        item = game.active_inventory
        item_type = item.item_type
        if item_type == "connection":
            start = item.targets[0]
            end = item.targets[1]
            if start is None and end is None:
                return
            if start is None:
                start_pos = game.cursor_pos + item.line[0]
                start = game.add_node(start_pos)
                if start is None:
                    return
            if end is None:
                end_pos = game.cursor_pos + item.line[1]
                end = game.add_node(end_pos)
                if end is None:
                    return
            game.connect_nodes(start, end, stiffness=10)
    if button == mouse.WHEEL_UP:
        game.rotate_active_inventory(-5.0)
    if button == mouse.WHEEL_DOWN:
        game.rotate_active_inventory(5.0)


def update(dt):
    if keyboard[keys.LEFT]:
        game.rotate_active_inventory(-45.0 * dt)
    if keyboard[keys.RIGHT]:
        game.rotate_active_inventory(45.0 * dt)
    game.update_positions(dt)
    game.update_agent_positions(dt)
    if game.finished is True:
        clock.schedule(load_next_level, 5.0)
        game.finished = "loading"


def draw():
    screen.clear()
    if game.tex1 is not None:
        for x in range(game.tex1.shape[1]):
            for y in range(game.tex1.shape[0]):
                tile_id = game.tex1[y, x]
                if tile_id == 0:
                    continue
                texture = TEXTURES[tile_id]
                pos = (
                    x * TILE_SIZE + (TILE_SIZE - texture.get_width()) / 2,
                    y * TILE_SIZE + (TILE_SIZE - texture.get_height()) / 2,
                )
                screen.blit(texture, pos)
    if False:  # game.flow is not None:
        for x in range(game.flow.shape[1]):
            for y in range(game.flow.shape[0]):
                tile_id = game.flow[y, x]
                if tile_id == 0:
                    continue
                texture = FLOW_TEXTURES[tile_id]
                pos = (
                    x * TILE_SIZE + (TILE_SIZE - texture.get_width()) / 2,
                    y * TILE_SIZE + (TILE_SIZE - texture.get_height()) / 2,
                )
                screen.blit(texture, pos)
    for i in range(game.next_particle):
        screen.draw.circle(game.particle_pos[i].astype(np.int32), 2, (255, 255, 255))
    bridge_segment = Actor("foliagepack022")
    if game.active_inventory:
        item = game.active_inventory
        item_type = item.item_type
        if item_type == "connection":
            start = game.cursor_pos + item.line[0]
            end = game.cursor_pos + item.line[1]
            bridge_segment.center = game.cursor_pos
            bridge_segment.angle = bridge_segment.angle_to(end)
            bridge_segment.draw()
            item.targets = []
            for pos in [start, end]:
                closest = game.find_closest_node(pos, CONNECT_DISTANCE)
                item.targets.append(closest)
                if closest is not None:
                    screen.blit(
                        images.platformpacktile024,
                        game.node_pos[closest].astype(np.int32) - 24,
                    )
    for i in range(game.next_node):
        screen.blit(images.nodesmall, game.node_pos[i].astype(np.int32) - 4)
    for i in range(game.next_edge):
        from_node, to_node = game.edges[i]
        from_pos = game.node_pos[from_node]
        to_pos = game.node_pos[to_node]
        screen.draw.line(
            from_pos.astype(np.int32), to_pos.astype(np.int32), (189, 137, 88)
        )
        bridge_segment.center = (from_pos + (to_pos - from_pos) / 2).astype(np.int32)
        bridge_segment.angle = bridge_segment.angle_to(to_pos.astype(np.int32))
        bridge_segment.draw()
    if game.next_edge == 0:
        screen.blit(
            images.platformpacktile024,
            game.node_pos[game.start_node].astype(np.int32) - 24,
        )
    screen.blit(
        START_NODE,
        game.node_pos[game.start_node]
        - np.array([START_NODE.get_width() / 2, START_NODE.get_height() - 10]),
    )
    for i in range(game.next_agent):
        tex = ANIMALS[game.agent_tex[i]]
        pos = (game.agent_pos[i] - np.array(tex.get_size()) / 2).astype(np.int32)
        screen.blit(tex, pos)
    screen.blit(
        GOAL,
        (np.array(game.target_marker) - np.array(GOAL.get_size()) / 2).astype(np.int32),
    )
    used_segments = game.next_edge
    agents_left = (game.agent_state != 4).sum()
    screen.draw.text("Animals left: {}".format(agents_left), (25, 25))
    screen.draw.text("Logs used: {}".format(used_segments), (25, 50))
    if game.finished:
        screen.draw.text("Finished!", fontsize=72, center=(WIDTH // 2, HEIGHT // 2))
        screen.draw.text(
            "{} logs   {:3d} seconds".format(game.next_edge, int(game.time)),
            fontsize=36,
            center=(WIDTH // 2, HEIGHT // 2 + 36),
        )
        if current_level >= len(LEVELS) - 1:
            screen.draw.text(
                "You completed the game. Thanks for playing!",
                fontsize=48,
                center=(WIDTH // 2, HEIGHT // 2 + 100),
            )
    elif game.time < 5:
        screen.draw.text(
            "Level {}".format(current_level + 1),
            fontsize=72,
            center=(WIDTH // 2, HEIGHT // 2),
        )


def load_next_level():
    global game, current_level
    current_level += 1
    if current_level < len(LEVELS):
        game = GameState(WIDTH, HEIGHT)
        game.load_map(LEVELS[current_level])
        game.active_inventory = InventoryItem(
            "connection", line=np.array([[-25, 25], [25, -25]]), targets=[None, None]
        )


pgzrun.go()
