import random
import xml.etree.ElementTree as et

from pgzero.loaders import ResourceLoader, images

import numpy as np


START_NODE = images.foliagepack023
GOAL = images.foliagepack010


TEXTURES = {
    201: images.rpgtile000,
    202: images.rpgtile001,
    203: images.rpgtile002,
    204: images.rpgtile003,
    205: images.rpgtile004,
    206: images.rpgtile005,
    207: images.rpgtile006,
    208: images.rpgtile007,
    209: images.rpgtile008,
    210: images.rpgtile009,
    211: images.rpgtile010,
    212: images.rpgtile011,
    213: images.rpgtile012,
    214: images.rpgtile013,
    215: images.rpgtile014,
    221: images.rpgtile018,
    222: images.rpgtile019,
    223: images.rpgtile020,
    224: images.rpgtile021,
    225: images.rpgtile022,
    226: images.rpgtile023,
    227: images.rpgtile024,
    228: images.rpgtile025,
    229: images.rpgtile026,
    230: images.rpgtile027,
    231: images.rpgtile028,
    232: images.rpgtile029,
    233: images.rpgtile030,
    234: images.rpgtile031,
    235: images.rpgtile032,
    241: images.rpgtile036,
    242: images.rpgtile037,
    243: images.rpgtile038,
    244: images.rpgtile039,
    245: images.rpgtile040,
    246: images.rpgtile041,
    247: images.rpgtile042,
    248: images.rpgtile043,
    251: images.rpgtile044,
    252: images.rpgtile045,
    253: images.rpgtile046,
}


FLOW_TEXTURES = {
    2: images.arrowleft,
    3: images.upleft,
    22: images.arrowdown,
    89: images.downright,
    109: images.downleft,
    161: images.arrowup,
    181: images.arrowright,
    182: images.upright,
}

ANIMALS = [
    images.bear,
    images.buffalo,
    images.chick,
    images.chicken,
    images.cow,
    images.dog,
    images.elephant,
    images.giraffe,
    images.goat,
    images.horse,
    images.monkey,
    images.moose,
    images.panda,
    images.pig,
    images.rabbit,
    images.rhino,
    images.sloth,
    images.zebra,
]

_SIN45 = np.sin(np.pi / 4)

FLOW_DIR = {
    0: np.array([0, 0]),
    2: np.array([-1, 0]),
    3: np.array([-_SIN45, -_SIN45]),
    22: np.array([0, 1]),
    89: np.array([_SIN45, _SIN45]),
    109: np.array([-_SIN45, _SIN45]),
    161: np.array([0, -1]),
    181: np.array([1, 0]),
    182: np.array([_SIN45, -_SIN45]),
}

ZONE_NONE = 0
ZONE_TARGET = 4


class MapLoader(ResourceLoader):
    EXTNS = ["tmx"]
    TYPE = "map"

    def _load(self, path):
        return et.parse(path).getroot()


maps = MapLoader("maps")


def get_layer(map_data, layer_name):
    layer = map_data.find(".//layer[@name='{}']".format(layer_name))
    size = int(layer.attrib["height"]), int(layer.attrib["width"])
    data = np.fromstring(layer.find("data").text, dtype=np.int, sep=",").reshape(size)
    return data


def fixed_nodes(map_data):
    nodes = map_data.findall(".//object[@type='fixed-node']")
    for node in nodes:
        yield (
            int(float(node.attrib["x"])),
            int(float(node.attrib["y"])),
        ), node.attrib.get("name") == "start"


def get_goal_pos(map_data):
    goal = map_data.find(".//object[@type='goal']")
    if goal:
        return int(float(goal.attrib["x"])), int(float(goal.attrib["y"]))


def random_starting_point(map_data, area_name="start-area"):
    areas = map_data.findall(".//object[@type='{}']".format(area_name))
    area = random.choice(areas)
    pos = np.array([float(area.attrib["x"]), float(area.attrib["y"])])
    pos += (
        np.random.rand(2)
        * np.array([float(area.attrib["width"]), float(area.attrib["height"])])
    ).astype(np.int32)
    return pos
